/*

	weplab - Wep Key Cracker

	Copyright (C) 2004 Jose Ignacio Sanchez Martin - Topo[LB]

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software Foundation,
	Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

	---------

	analpfile.c: dictionary based wep crack.
*/

#include <pcap.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <time.h>

#include "wep.h"
#include "globals.h"
#include "debug.h"
#include "analpfile.h"
#include "heuristics.h"


#define HUGE_MEM_WEAKPACKETS_SIZE	104000000

void AnalyzePcapFile(void){
	t_StoredPacketsIV packetsIV;
	pcap_t *file;
	unsigned long int totalPackets=0;

	unsigned long int fileSize=0;
	unsigned long int actualFilePosition;

	unsigned long int perc;
	unsigned long int lastFilePosition=0;
	unsigned char * pkt;
	struct pcap_pkthdr h;
	t_wi_frame *wi_h; 
	unsigned char *iv;

	unsigned long lastProgressPerc=0;
	
	unsigned long int totalValidPackets=0;
	unsigned long int totalUniqIV=0;
	unsigned long int totalPacketsTruncated=0;
	unsigned long int totalPacketsNonData=0;
	unsigned long int totalPacketsChecksumFF=0;


	unsigned char *actual_iv;
	FILE * fileDescriptor;



	#ifdef _DEBUG
	debug("Starting pcap file analysis.");
	#endif

	packetsIV.iv=malloc(HUGE_MEM_WEAKPACKETS_SIZE);
	packetsIV.len=0;
	if (!packetsIV.iv) {
		printf("Error: not enough memmory to allocate %u",HUGE_MEM_WEAKPACKETS_SIZE);	
		exit (-1);
	}

	debug ("Setting the memmory to 0s");
	memset(packetsIV.iv,0,HUGE_MEM_WEAKPACKETS_SIZE);
	
	debug ("Opening packet file for reading sample encrypted packets");
	file=OpenPcapFile(global_v.packetsFilename);	

	fileDescriptor=pcap_file(file);
	actualFilePosition=ftell(fileDescriptor);
	fseek(fileDescriptor,0,SEEK_END);
	fileSize=ftell(fileDescriptor);
	fseek(fileDescriptor,actualFilePosition,SEEK_SET);


	perc=fileSize/100;

	while(1)
	{
	if ((pkt = (unsigned char *) pcap_next(file, &h)) == NULL) break;
	totalPackets++;

	actualFilePosition=ftell(fileDescriptor);
	if (actualFilePosition-lastFilePosition>perc) {
		printf("\r %lu %% read",lastProgressPerc++);
		fflush(stdout);
		lastFilePosition=actualFilePosition;
	}

	if (global_v.prismHeaderPresent) pkt+=144;
    wi_h=(t_wi_frame *)pkt;

// Here do some sanity checks with the 802.11 header
  
  if (h.len!=h.caplen) totalPacketsTruncated++;
	
	if(!(wi_h->wi_frame_ctl&0x08)) totalPacketsNonData++;
	else
	{
		//p_len=h.caplen;
		if (((*(((unsigned char *)wi_h)+1))&0xC0)==0xC0) iv=pkt+sizeof(t_wi_frame)+6;
		else iv=pkt+sizeof(t_wi_frame);

		if (global_v.debug>1){ 
			debug("Packet %u",totalPackets);
			debug("---------------------------------------------");
			DebugViewPacketHeaders(pkt);
			debug("Pcap Len: %u",h.len);
			debug("Key ID: %u",(unsigned int)iv[3]);			
			debug("---------------------------------------------\n");
		}
		totalValidPackets++;


		if (global_v.debug>1) DebugViewKey(iv,3);			
		actual_iv=packetsIV.iv+((unsigned int)(((unsigned int) iv[0]) | (((unsigned int) iv[1]) << 8) | (((unsigned int) iv[2]) << 16)));
				if (!actual_iv[0]){
					totalUniqIV++;
				}
		
		if (global_v.fcsPresent) if (iv[h.len-4-1]==0xFF &&iv[h.len-4]==0xFF &&iv[h.len-4+1]==0xFF &&iv[h.len-4+2]==0xFF) totalPacketsChecksumFF++;
		
		if (!global_v.fcsPresent) if (iv[h.len-1]==0xFF &&iv[h.len]==0xFF &&iv[h.len+1]==0xFF &&iv[h.len+2]==0xFF) totalPacketsChecksumFF++;
	}

	}
printf("\r                                   \n");

printf("Total valid packets read: %lu\n",totalValidPackets);
printf("Total packets read: %lu\n",totalPackets);
printf("Total unique IV read: %lu\n",totalUniqIV);
printf("Total truncated packets read: %lu\n",totalPacketsTruncated);
printf("Total non-data packets read: %lu\n",totalPacketsNonData);
printf("Total FF checksum packets read: %lu\n",totalPacketsChecksumFF);

	
ClosePcapFile(file);
	
}
